import {
  flush,
  getDefaultIsolationScope,
  getIsolationScope,
  GLOBAL_OBJ,
  logger,
  vercelWaitUntil,
  withIsolationScope
} from "@sentry/core";
import * as SentryNode from "@sentry/node";
import { H3Error } from "h3";
import { defineNitroPlugin } from "nitropack/runtime";
import { addSentryTracingMetaTags, extractErrorContext } from "../utils.js";
export default defineNitroPlugin((nitroApp) => {
  nitroApp.h3App.handler = patchEventHandler(nitroApp.h3App.handler);
  nitroApp.hooks.hook("error", async (error, errorContext) => {
    const sentryClient = SentryNode.getClient();
    const sentryClientOptions = sentryClient?.getOptions();
    if (sentryClientOptions && "enableNitroErrorHandler" in sentryClientOptions && sentryClientOptions.enableNitroErrorHandler === false) {
      return;
    }
    if (error instanceof H3Error) {
      if (error.statusCode >= 300 && error.statusCode < 500) {
        return;
      }
    }
    const { method, path } = {
      method: errorContext.event?._method ? errorContext.event._method : "",
      path: errorContext.event?._path ? errorContext.event._path : null
    };
    if (path) {
      SentryNode.getCurrentScope().setTransactionName(`${method} ${path}`);
    }
    const structuredContext = extractErrorContext(errorContext);
    SentryNode.captureException(error, {
      captureContext: { contexts: { nuxt: structuredContext } },
      mechanism: { handled: false }
    });
    await flushIfServerless();
  });
  nitroApp.hooks.hook("render:html", (html) => {
    addSentryTracingMetaTags(html.head);
  });
});
async function flushIfServerless() {
  const isServerless = !!process.env.FUNCTIONS_WORKER_RUNTIME || // Azure Functions
  !!process.env.LAMBDA_TASK_ROOT || // AWS Lambda
  !!process.env.VERCEL || !!process.env.NETLIFY;
  if (GLOBAL_OBJ[Symbol.for("@vercel/request-context")]) {
    vercelWaitUntil(flushWithTimeout());
  } else if (isServerless) {
    await flushWithTimeout();
  }
}
async function flushWithTimeout() {
  const sentryClient = SentryNode.getClient();
  const isDebug = sentryClient ? sentryClient.getOptions().debug : false;
  try {
    isDebug && logger.log("Flushing events...");
    await flush(2e3);
    isDebug && logger.log("Done flushing events");
  } catch (e) {
    isDebug && logger.log("Error while flushing events:\n", e);
  }
}
function patchEventHandler(handler) {
  return new Proxy(handler, {
    async apply(handlerTarget, handlerThisArg, handlerArgs) {
      const isolationScope = getIsolationScope();
      const newIsolationScope = isolationScope === getDefaultIsolationScope() ? isolationScope.clone() : isolationScope;
      logger.log(
        `Patched h3 event handler. ${isolationScope === newIsolationScope ? "Using existing" : "Created new"} isolation scope.`
      );
      return withIsolationScope(newIsolationScope, async () => {
        try {
          return await handlerTarget.apply(handlerThisArg, handlerArgs);
        } finally {
          await flushIfServerless();
        }
      });
    }
  });
}
